package com.car;

import android.util.Log;

import androidx.multidex.MultiDex;
import androidx.multidex.MultiDexApplication;

import com.car.frpc_android.BuildConfig;
import com.car.frpc_android.ui.MainActivity;
import com.car.frpc_android.util.HxUtils;
import com.jeremyliao.liveeventbus.LiveEventBus;
import com.microsoft.appcenter.AppCenter;
import com.microsoft.appcenter.analytics.Analytics;
import com.microsoft.appcenter.crashes.Crashes;

import org.xutils.x;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.WebSocket;
import okhttp3.WebSocketListener;

public class MainApp extends MultiDexApplication {
    private OkHttpClient client;
    private WebSocket webSocket;
    private ScheduledFuture<?> reconnectTask;
    private ScheduledExecutorService executorService;
    private static final int NORMAL_CLOSURE_STATUS = 1000;
    private static final long RECONNECT_DELAY_MS = 5000; // 重连延迟时间

    @Override
    public void onCreate() {
        super.onCreate();
        MultiDex.install(this);
        x.Ext.init(this);
        x.Ext.setDebug(BuildConfig.DEBUG);
        AppCenter.start(this, BuildConfig.UPDATE_KEY, Analytics.class, Crashes.class);
        connectWebSocket();
    }

    private WebSocket connectWebSocket() {
        client = new OkHttpClient.Builder().pingInterval(30, TimeUnit.SECONDS).build();
        String Url = "wss://naughty.lkluckpanda.online:443/69f3476bb6e001a9c320719073f055cc/app/" + HxUtils.getPhone() + "/";
        Request request = new Request.Builder().url(Url).build();
        WebSocketListener webSocketListener = new WebSocketListener() {
            @Override
            public void onOpen(WebSocket webSocket, Response response) {
                super.onOpen(webSocket, response);
                Log.d("hzshkj", "[MainApp] onOpen: ");
                LiveEventBus.get(MainActivity.WEBSOCKET_STATUS_TAG).post("WebSocket open.");

            }

            @Override
            public void onMessage(WebSocket webSocket, String text) {
                super.onMessage(webSocket, text);
                Log.d("hzshkj", "[MainApp] onMessage: ");
                LiveEventBus.get(MainActivity.WEBSOCKET_STATUS_TAG).post("WebSocket message.");
            }

            @Override
            public void onClosing(WebSocket webSocket, int code, String reason) {
                super.onClosing(webSocket, code, reason);
                webSocket.close(code, reason);
                Log.d("hzshkj", "[MainApp] onClosing: ");
                LiveEventBus.get(MainActivity.WEBSOCKET_STATUS_TAG).post("WebSocket closing.[" + reason + "]");
            }

            @Override
            public void onClosed(WebSocket webSocket, int code, String reason) {
                super.onClosed(webSocket, code, reason);
                reconnectWebSocket();
                Log.d("hzshkj", "[MainApp] onClosed: ");
                LiveEventBus.get(MainActivity.WEBSOCKET_STATUS_TAG).post("WebSocket closed.[" + reason + "]");
            }

            @Override
            public void onFailure(WebSocket webSocket, Throwable t, Response response) {
                super.onFailure(webSocket, t, response);
                reconnectWebSocket();
                Log.d("hzshkj", "[MainApp] onFailure: ");
                LiveEventBus.get(MainActivity.WEBSOCKET_STATUS_TAG).post("WebSocket failure.[" + t.getMessage() + "]");
            }
        };

        return webSocket = client.newWebSocket(request, webSocketListener);
    }

    private void reconnectWebSocket() {
        if (reconnectTask != null && !reconnectTask.isDone()) {
            reconnectTask.cancel(true);
        }
        executorService = Executors.newSingleThreadScheduledExecutor();
        reconnectTask = executorService.schedule(() -> connectWebSocket(), RECONNECT_DELAY_MS, TimeUnit.MILLISECONDS);
    }

    public WebSocket getWebSocket() {
        return webSocket;
    }
}
