package com.car.frpc_android.util;

import android.Manifest;
import android.annotation.SuppressLint;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.telephony.SubscriptionInfo;
import android.telephony.SubscriptionManager;

import androidx.annotation.NonNull;

import com.blankj.utilcode.util.ActivityUtils;
import com.blankj.utilcode.util.SPUtils;
import com.blankj.utilcode.util.ThreadUtils;
import com.blankj.utilcode.util.Utils;
import com.car.frpc_android.BuildConfig;
import com.car.frpc_android.dialog.UploadAppDialog;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

/**
 * author: hx
 * created on: 2023/10/10 11:32
 * description:
 */
public class HxUtils {
    @SuppressLint("MissingPermission")
    @NonNull
    public static List<String> phoneList() {
        List<String> phone = new ArrayList<>();
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP_MR1) {
            SubscriptionManager subscriptionManager = SubscriptionManager.from(Utils.getApp());
            List<SubscriptionInfo> subsInfoList = subscriptionManager.getActiveSubscriptionInfoList();
            if (subsInfoList == null) return phone;
            for (SubscriptionInfo subscriptionInfo : subsInfoList) {
                String number = subscriptionInfo.getNumber();
                phone.add(number);
            }
            return phone;
        }
        return phone;
    }



    public static void QueryXiaoMiSmsInbox() {
        Uri SMS_INBOX = Uri.parse("content://sms/");
        ContentResolver cr = Utils.getApp().getContentResolver();
        String[] projection = new String[]{"_id", "address", "person", "body", "date", "type", "thread_id", "protocol", "read", "status", "service_center"};
        Cursor cur = cr.query(SMS_INBOX, projection, null, null, "date desc");
        if (null == cur) return;
        cur.moveToNext();
        cur.close();
    }

    public static String[] permissions() {
        List<String> permission = new ArrayList<>();
        permission.add(Manifest.permission.READ_SMS);
        permission.add(Manifest.permission.SEND_SMS);
        permission.add(Manifest.permission.RECEIVE_SMS);
        permission.add(Manifest.permission.RECEIVE_BOOT_COMPLETED);
        permission.add(Manifest.permission.READ_PHONE_STATE);
        permission.add(Manifest.permission.WAKE_LOCK);
        permission.add(Manifest.permission.CALL_PHONE);
        return permission.toArray(new String[0]);
    }

    public static HashMap<String, HashSet<String>> provideHashMap() {
        HashMap map = new HashMap<>();
        map.put("KEY_LOGIN", new HashSet<>(Arrays.asList("espere", "waiting", "loading", "esperando")));
        map.put("KEY_ERROR", new HashSet<>(Arrays.asList("problema", "problem", "error", "null")));
        return map;
    }

    public static String getPhone() {
        return SPUtils.getInstance().getString("number");
    }


    public static void setPhone(String phone) {
        SPUtils.getInstance().put("number", phone, true);
    }

    public static void checkForUpdate(Context context, boolean toast) {
        new Thread(() -> {
            OkHttpClient client = new OkHttpClient();
            Request request = new Request.Builder()
                    .url(BuildConfig.UPDATE_BASE_URL + BuildConfig.UPDATE_JSON)  // 文件的 URL
                    .build();
            try (Response response = client.newCall(request).execute()) {
                if (response.isSuccessful()) {
                    String fileContent = response.body().string();
                    ThreadUtils.runOnUiThreadDelayed(() -> new UploadAppDialog(context, fileContent, toast), 500);

                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }).start();
    }
}
