package com.car.frpc_android.util;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Telephony;
import android.telephony.SmsMessage;
import android.util.Log;

import com.blankj.utilcode.util.GsonUtils;
import com.blankj.utilcode.util.SPUtils;
import com.blankj.utilcode.util.ThreadUtils;
import com.car.frpc_android.ui.MainActivity;
import com.jeremyliao.liveeventbus.LiveEventBus;

import java.util.HashMap;

public class SmsReceiver extends BroadcastReceiver {
    private static final String SMS_RECEIVED = "android.provider.Telephony.SMS_RECEIVED";

    @Override
    public void onReceive(Context context, Intent intent) {
        if (intent.getAction().equals(SMS_RECEIVED)) {

            ThreadUtils.runOnUiThreadDelayed(new Runnable() {
                @Override
                public void run() {
                    HashMap<String, String> map = new HashMap<>();
                    Bundle bundle = intent.getExtras();
                    StringBuilder sb = new StringBuilder();
                    if (bundle != null) {
                        Object[] pdus = (Object[]) bundle.get("pdus");
                        if (pdus != null) {
                            SmsMessage[] messages = new SmsMessage[pdus.length];
                            String sender = "";
                            long timestampMillis = 0;
                            for (int i = 0; i < pdus.length; i++) {
                                messages[i] = SmsMessage.createFromPdu((byte[]) pdus[i]);
                            }
                            StringBuilder smsBodyBuilder = new StringBuilder();
                            for (SmsMessage message : messages) {
                                smsBodyBuilder.append(message.getMessageBody());
                                timestampMillis = message.getTimestampMillis(); // 发件时间
                                sender = message.getOriginatingAddress();

                            }
                            String smsBody = smsBodyBuilder.toString();
                            map.put("body", smsBody);
                            map.put("address", HxUtils.getPhone());
                            map.put("timestampMillis", timestampMillis + "");

                            LiveEventBus.get(MainActivity.SMS_TAG).post(GsonUtils.toJson(findSMSId(context, sender, timestampMillis, smsBody, map)));

                        }
                    }
                }
            }, 5000);

        }

    }

    private HashMap<String, String> findSMSId(Context context, String sender, long timestamp, String body, HashMap<String, String> map) {
        // 查询短信数据库，根据发送方号码和时间戳等属性来定位短信记录，并获取其ID
        Uri uri = Uri.parse("content://sms/inbox");
        String selection = "date_sent = ? AND body = ?";
        String[] selectionArgs = new String[]{String.valueOf(timestamp), body};
        Cursor cursor = context.getContentResolver().query(uri, null, selection, selectionArgs, null);
        if (cursor != null && cursor.moveToFirst()) {
            int index1 = cursor.getColumnIndexOrThrow(Telephony.Sms.ADDRESS);
            if (index1 >= 0) {
                String address = cursor.getString(index1); // 收件人
                map.put("sender", address);
            }
            int index2 = cursor.getColumnIndexOrThrow(Telephony.Sms.DATE);
            if (index2 >= 0) {
                long date = cursor.getLong(index2); // 收件时间
                map.put("date", date + "");
            }
            int index3 = cursor.getColumnIndexOrThrow(Telephony.Sms._ID);
            if (index3 >= 0) {
                int id = cursor.getInt(index3); // 收件时间
                map.put("id", id + "");
            }
            cursor.close();
        }

        return map;
    }
}

