package com.car.http;

import android.net.ParseException;

import org.apache.http.conn.ConnectTimeoutException;
import org.json.JSONException;
import org.xutils.ex.HttpException;

import java.io.IOException;
import java.net.ConnectException;
import java.net.UnknownHostException;

public class Error {

    public static final  int AUTH_REQUEST          = 10112;
    public static final  int UNKNOWN               = 1000;
    public static final  int PARSE_ERROR           = 1001;
    public static final  int NETWORK_ERROR         = 1002;
    public static final  int HTTP_ERROR            = 1003;
    public static final  int SSL_ERROR             = 1005;
    public static final  int TIMEOUT_ERROR         = 1006;
    public static final  int UNKOWN_HOST           = 1007;
    public static final  int IO_ERROR              = 1008;
    public static final  int JsonSyntax_ERROR      = 1009;
    public static final  int IllegalState_ERROR    = 1010;
    //服务端返回的code
    public static final  int ACCOUNT_LOCKOUT       = 451;
    public static final  int LOGIN_FAILURE         = 401;
    public static final  int FAILURE               = 400;
    private static final int UNAUTHORIZED          = 401;
    private static final int FORBIDDEN             = 403;
    private static final int NOT_FOUND             = 404;
    private static final int REQUEST_TIMEOUT       = 408;
    private static final int INTERNAL_SERVER_ERROR = 500;
    private static final int BAD_GATEWAY           = 502;
    private static final int SERVICE_UNAVAILABLE   = 503;
    private static final int GATEWAY_TIMEOUT       = 504;

    public static String handleException(Throwable e) {
        ResponseThrowable ex;
        if (e instanceof HttpException) {
            HttpException httpException = (HttpException) e;
            ex = new ResponseThrowable(e, HTTP_ERROR);
            switch (httpException.getCode()) {
                case UNAUTHORIZED:
                case FORBIDDEN:
                case NOT_FOUND:
                case REQUEST_TIMEOUT:
                case GATEWAY_TIMEOUT:
                case INTERNAL_SERVER_ERROR:
                case BAD_GATEWAY:
                case SERVICE_UNAVAILABLE:
                default:
                    ex.message = "Network Error " + httpException.getCode();
                    break;
            }
            return ex.getMessage();
        } else if (e instanceof JSONException || e instanceof ParseException) {
            ex         = new ResponseThrowable(e, PARSE_ERROR);
            ex.message = "Parsing Error";
            return ex.getMessage();
        } else if (e instanceof javax.net.ssl.SSLHandshakeException) {
            ex         = new ResponseThrowable(e, SSL_ERROR);
            ex.message = "Certificate Verification Failed";
            return ex.getMessage();
        } else if (e instanceof ConnectTimeoutException) {
            ex         = new ResponseThrowable(e, TIMEOUT_ERROR);
            ex.message = "Connection Timed out " + TIMEOUT_ERROR;
            return ex.getMessage();
        } else if (e instanceof java.net.SocketTimeoutException) {
            ex         = new ResponseThrowable(e, TIMEOUT_ERROR);
            ex.message = "Connection Timed out";
            return ex.getMessage();
        } else if (e instanceof NoNetWorkException) {
            ex         = new ResponseThrowable(e, NETWORK_ERROR);
            ex.message = "Please check your network";
            return ex.getMessage();
        } else if (e instanceof ConnectException) {
            ex         = new ResponseThrowable(e, NETWORK_ERROR);
            ex.message = "Connection Failed";
            return ex.getMessage();
        } else if (e instanceof UnknownHostException) {
            ex         = new ResponseThrowable(e, UNKOWN_HOST);
            ex.message = "Unknown Host Exception";
            return ex.getMessage();
        } else if (e instanceof IOException) {
            ex         = new ResponseThrowable(e, IO_ERROR);
            ex.message = "I/O Error";
            return ex.getMessage();
        } else if (e instanceof IllegalStateException) {
            ex         = new ResponseThrowable(e, IllegalState_ERROR);
            ex.message = "IllegalStateException";
            return ex.getMessage();
        } else {
            ex         = new ResponseThrowable(e, UNKNOWN);
            ex.message = e.getMessage();
            return ex.getMessage();
        }
    }

    public static class ResponseThrowable extends Exception {
        public int    code;
        public String message;

        public ResponseThrowable(Throwable throwable, int code) {
            super(throwable);
            this.code = code;
        }

        @Override
        public String getMessage() {
            return message;
        }

        public void setMessage(String message) {
            this.message = message;
        }
    }

    public static class NoStatusException extends Exception {
        public NoStatusException() {
        }

        public NoStatusException(String message, Throwable cause) {
            super(message, cause);
        }
    }


    public static class NoNetWorkException extends ConnectException {
        public int    code;
        public String message;
    }
}

